@@ -1,11 +1,20 @@
+from rope.base.pynames import DefinedName
 import rope.base.builtins
 import rope.base.codeanalyze
 import rope.base.evaluate
 import rope.base.libutils
 import rope.base.oi.soi
 import rope.base.pyscopes
-from rope.base import (pynamesdef as pynames, exceptions, ast,
-                       astutils, pyobjects, fscommands, arguments, utils)
+from rope.base import (
+    pynamesdef as pynames,
+    exceptions,
+    ast,
+    astutils,
+    pyobjects,
+    fscommands,
+    arguments,
+    utils,
+)
 from rope.base.utils import pycompat
 
 try:
@@ -15,14 +24,13 @@ except NameError:
 
 
 class PyFunction(pyobjects.PyFunction):
-
     def __init__(self, pycore, ast_node, parent):
         rope.base.pyobjects.AbstractFunction.__init__(self)
-        rope.base.pyobjects.PyDefinedObject.__init__(
-            self, pycore, ast_node, parent)
+        rope.base.pyobjects.PyDefinedObject.__init__(self, pycore, ast_node, parent)
         self.arguments = self.ast_node.args
         self.parameter_pyobjects = pynames._Inferred(
-            self._infer_parameters, self.get_module()._get_concluded_data())
+            self._infer_parameters, self.get_module()._get_concluded_data()
+        )
         self.returned = pynames._Inferred(self._infer_returned)
         self.parameter_pynames = None
 
@@ -33,8 +41,7 @@ class PyFunction(pyobjects.PyFunction):
         return {}
 
     def _create_scope(self):
-        return rope.base.pyscopes.FunctionScope(self.pycore, self,
-                                                _FunctionVisitor)
+        return rope.base.pyscopes.FunctionScope(self.pycore, self, _FunctionVisitor)
 
     def _infer_parameters(self):
         pyobjects = rope.base.oi.soi.infer_parameter_objects(self)
@@ -77,8 +84,11 @@ class PyFunction(pyobjects.PyFunction):
 
     def get_param_names(self, special_args=True):
         # TODO: handle tuple parameters
-        result = [pycompat.get_ast_arg_arg(node) for node in self.arguments.args
-                  if isinstance(node, pycompat.ast_arg_type)]
+        result = [
+            pycompat.get_ast_arg_arg(node)
+            for node in self.arguments.args
+            if isinstance(node, pycompat.ast_arg_type)
+        ]
         if special_args:
             if self.arguments.vararg:
                 result.append(pycompat.get_ast_arg_arg(self.arguments.vararg))
@@ -97,28 +107,38 @@ class PyFunction(pyobjects.PyFunction):
         if isinstance(self.parent, PyClass):
             for decorator in self.decorators:
                 pyname = rope.base.evaluate.eval_node(scope, decorator)
-                if pyname == rope.base.builtins.builtins['staticmethod']:
-                    return 'staticmethod'
-                if pyname == rope.base.builtins.builtins['classmethod']:
-                    return 'classmethod'
-            return 'method'
-        return 'function'
+                if pyname == rope.base.builtins.builtins["staticmethod"]:
+                    return "staticmethod"
+                if pyname == rope.base.builtins.builtins["classmethod"]:
+                    return "classmethod"
+            return "method"
+        return "function"
 
     @property
     def decorators(self):
         try:
-            return getattr(self.ast_node, 'decorator_list')
+            return getattr(self.ast_node, "decorator_list")
         except AttributeError:
-            return getattr(self.ast_node, 'decorators', None)
+            return getattr(self.ast_node, "decorators", None)
+
+
+class PyComprehension(pyobjects.PyComprehension):
+    def __init__(self, pycore, ast_node, parent):
+        self.visitor_class = _ComprehensionVisitor
+        rope.base.pyobjects.PyObject.__init__(self, type_="Comp")
+        rope.base.pyobjects.PyDefinedObject.__init__(self, pycore, ast_node, parent)
+
+    def _create_scope(self):
+        return rope.base.pyscopes.ComprehensionScope(
+            self.pycore, self, _ComprehensionVisitor
+        )
 
 
 class PyClass(pyobjects.PyClass):
-
     def __init__(self, pycore, ast_node, parent):
         self.visitor_class = _ClassVisitor
         rope.base.pyobjects.AbstractClass.__init__(self)
-        rope.base.pyobjects.PyDefinedObject.__init__(
-            self, pycore, ast_node, parent)
+        rope.base.pyobjects.PyDefinedObject.__init__(self, pycore, ast_node, parent)
         self.parent = parent
         self._superclasses = self.get_module()._get_concluded_data()
 
@@ -139,12 +159,13 @@ class PyClass(pyobjects.PyClass):
     def _get_bases(self):
         result = []
         for base_name in self.ast_node.bases:
-            base = rope.base.evaluate.eval_node(self.parent.get_scope(),
-                                                base_name)
-            if base is not None and \
-                base.get_object().get_type() == \
-                    rope.base.pyobjects.get_base_type('Type'):
-                    result.append(base.get_object())
+            base = rope.base.evaluate.eval_node(self.parent.get_scope(), base_name)
+            if (
+                base is not None
+                and base.get_object().get_type()
+                == rope.base.pyobjects.get_base_type("Type")
+            ):
+                result.append(base.get_object())
         return result
 
     def _create_scope(self):
@@ -152,10 +173,8 @@ class PyClass(pyobjects.PyClass):
 
 
 class PyModule(pyobjects.PyModule):
-
-    def __init__(self, pycore, source=None,
-                 resource=None, force_errors=False):
-        ignore = pycore.project.prefs.get('ignore_syntax_errors', False)
+    def __init__(self, pycore, source=None, resource=None, force_errors=False):
+        ignore = pycore.project.prefs.get("ignore_syntax_errors", False)
         syntax_errors = force_errors or not ignore
         self.has_errors = False
         try:
@@ -165,8 +184,8 @@ class PyModule(pyobjects.PyModule):
             if syntax_errors:
                 raise
             else:
-                source = '\n'
-                node = ast.parse('\n')
+                source = "\n"
+                node = ast.parse("\n")
         self.source_code = source
         self.star_imports = []
         self.visitor_class = _GlobalVisitor
@@ -174,7 +193,7 @@ class PyModule(pyobjects.PyModule):
         super(PyModule, self).__init__(pycore, node, resource)
 
     def _init_source(self, pycore, source_code, resource):
-        filename = 'string'
+        filename = "string"
         if resource:
             filename = resource.path
         try:
@@ -190,7 +209,7 @@ class PyModule(pyobjects.PyModule):
         except SyntaxError as e:
             raise exceptions.ModuleSyntaxError(filename, e.lineno, e.msg)
         except UnicodeDecodeError as e:
-            raise exceptions.ModuleSyntaxError(filename, 1, '%s' % (e.reason))
+            raise exceptions.ModuleSyntaxError(filename, 1, "%s" % (e.reason))
         return source_code, ast_node
 
     @utils.prevent_recursion(lambda: {})
@@ -218,16 +237,17 @@ class PyModule(pyobjects.PyModule):
     def get_name(self):
         return rope.base.libutils.modname(self.get_resource())
 
-class PyPackage(pyobjects.PyPackage):
 
+class PyPackage(pyobjects.PyPackage):
     def __init__(self, pycore, resource=None, force_errors=False):
         self.resource = resource
         init_dot_py = self._get_init_dot_py()
         if init_dot_py is not None:
             ast_node = pycore.project.get_pymodule(
-                init_dot_py, force_errors=force_errors).get_ast()
+                init_dot_py, force_errors=force_errors
+            ).get_ast()
         else:
-            ast_node = ast.parse('\n')
+            ast_node = ast.parse("\n")
         super(PyPackage, self).__init__(pycore, ast_node, resource)
 
     def _create_structural_attributes(self):
@@ -255,16 +275,14 @@ class PyPackage(pyobjects.PyPackage):
         for child in self.resource.get_children():
             if child.is_folder():
                 result[child.name] = child
-            elif child.name.endswith('.py') and \
-                    child.name != '__init__.py':
+            elif child.name.endswith(".py") and child.name != "__init__.py":
                 name = child.name[:-3]
                 result[name] = child
         return result
 
     def _get_init_dot_py(self):
-        if self.resource is not None and \
-                self.resource.has_child('__init__.py'):
-            return self.resource.get_child('__init__.py')
+        if self.resource is not None and self.resource.has_child("__init__.py"):
+            return self.resource.get_child("__init__.py")
         else:
             return None
 
@@ -279,7 +297,6 @@ class PyPackage(pyobjects.PyPackage):
 
 
 class _AnnAssignVisitor(object):
-
     def __init__(self, scope_visitor):
         self.scope_visitor = scope_visitor
         self.assigned_ast = None
@@ -295,9 +312,9 @@ class _AnnAssignVisitor(object):
         self.scope_visitor._assigned(name, assignment)
 
     def _Name(self, node):
-        assignment = pynames.AssignmentValue(self.assigned_ast,
-                                             assign_type=True,
-                                             type_hint=self.type_hint)
+        assignment = pynames.AssignmentValue(
+            self.assigned_ast, assign_type=True, type_hint=self.type_hint
+        )
         self._assigned(node.id, assignment)
 
     def _Tuple(self, node):
@@ -321,8 +338,34 @@ class _AnnAssignVisitor(object):
         pass
 
 
-class _AssignVisitor(object):
+class _ExpressionVisitor(object):
+    def __init__(self, scope_visitor):
+        self.scope_visitor = scope_visitor
 
+    def _assigned(self, name, assignment=None):
+        self.scope_visitor._assigned(name, assignment)
+
+    def _GeneratorExp(self, node):
+        list_comp = PyComprehension(
+            self.scope_visitor.pycore, node, self.scope_visitor.owner_object
+        )
+        self.scope_visitor.defineds.append(list_comp)
+
+    def _SetComp(self, node):
+        self._GeneratorExp(node)
+
+    def _ListComp(self, node):
+        self._GeneratorExp(node)
+
+    def _DictComp(self, node):
+        self._GeneratorExp(node)
+
+    def _NamedExpr(self, node):
+        ast.walk(node.target, _AssignVisitor(self))
+        ast.walk(node.value, self)
+
+
+class _AssignVisitor(object):
     def __init__(self, scope_visitor):
         self.scope_visitor = scope_visitor
         self.assigned_ast = None
@@ -331,6 +374,7 @@ class _AssignVisitor(object):
         self.assigned_ast = node.value
         for child_node in node.targets:
             ast.walk(child_node, self)
+        ast.walk(node.value, _ExpressionVisitor(self.scope_visitor))
 
     def _assigned(self, name, assignment=None):
         self.scope_visitor._assigned(name, assignment)
@@ -359,9 +403,9 @@ class _AssignVisitor(object):
         pass
 
 
-class _ScopeVisitor(object):
-
+class _ScopeVisitor(_ExpressionVisitor):
     def __init__(self, pycore, owner_object):
+        _ExpressionVisitor.__init__(self, scope_visitor=self)
         self.pycore = pycore
         self.owner_object = owner_object
         self.names = {}
@@ -381,17 +425,23 @@ class _ScopeVisitor(object):
     def _FunctionDef(self, node):
         pyfunction = PyFunction(self.pycore, node, self.owner_object)
         for decorator in pyfunction.decorators:
-            if isinstance(decorator, ast.Name) and decorator.id == 'property':
+            if isinstance(decorator, ast.Name) and decorator.id == "property":
                 if isinstance(self, _ClassVisitor):
                     type_ = rope.base.builtins.Property(pyfunction)
                     arg = pynames.UnboundName(
-                        rope.base.pyobjects.PyObject(self.owner_object))
+                        rope.base.pyobjects.PyObject(self.owner_object)
+                    )
 
                     def _eval(type_=type_, arg=arg):
                         return type_.get_property_object(
-                            arguments.ObjectArguments([arg]))
+                            arguments.ObjectArguments([arg])
+                        )
+
+                    lineno = utils.guess_def_lineno(self.get_module(), node)
+
                     self.names[node.name] = pynames.EvaluatedName(
-                        _eval, module=self.get_module(), lineno=node.lineno)
+                        _eval, module=self.get_module(), lineno=lineno
+                    )
                     break
         else:
             self.names[node.name] = pynames.DefinedName(pyfunction)
@@ -410,8 +460,9 @@ class _ScopeVisitor(object):
         pass
 
     def _For(self, node):
-        names = self._update_evaluated(node.target, node.iter,  # noqa
-                                       '.__iter__().next()')
+        names = self._update_evaluated(
+            node.target, node.iter, ".__iter__().next()"  # noqa
+        )
         for child in node.body + node.orelse:
             ast.walk(child, self)
 
@@ -427,26 +478,28 @@ class _ScopeVisitor(object):
                 pyname.assignments.append(assignment)
             self.names[name] = pyname
 
-    def _update_evaluated(self, targets, assigned,
-                          evaluation='', eval_type=False, type_hint=None):
+    def _update_evaluated(
+        self, targets, assigned, evaluation="", eval_type=False, type_hint=None
+    ):
         result = {}
         if isinstance(targets, str):
-            assignment = pynames.AssignmentValue(assigned, [],
-                                                 evaluation, eval_type)
+            assignment = pynames.AssignmentValue(assigned, [], evaluation, eval_type)
             self._assigned(targets, assignment)
         else:
             names = astutils.get_name_levels(targets)
             for name, levels in names:
-                assignment = pynames.AssignmentValue(assigned, levels,
-                                                     evaluation, eval_type)
+                assignment = pynames.AssignmentValue(
+                    assigned, levels, evaluation, eval_type
+                )
                 self._assigned(name, assignment)
         return result
 
     def _With(self, node):
         for item in pycompat.get_ast_with_items(node):
             if item.optional_vars:
-                self._update_evaluated(item.optional_vars,
-                                       item.context_expr, '.__enter__()')
+                self._update_evaluated(
+                    item.optional_vars, item.context_expr, ".__enter__()"
+                )
         for child in node.body:
             ast.walk(child, self)
 
@@ -471,15 +524,13 @@ class _ScopeVisitor(object):
         for import_pair in node.names:
             module_name = import_pair.name
             alias = import_pair.asname
-            first_package = module_name.split('.')[0]
+            first_package = module_name.split(".")[0]
             if alias is not None:
-                imported = pynames.ImportedModule(self.get_module(),
-                                                  module_name)
+                imported = pynames.ImportedModule(self.get_module(), module_name)
                 if not self._is_ignored_import(imported):
                     self.names[alias] = imported
             else:
-                imported = pynames.ImportedModule(self.get_module(),
-                                                  first_package)
+                imported = pynames.ImportedModule(self.get_module(), first_package)
                 if not self._is_ignored_import(imported):
                     self.names[first_package] = imported
 
@@ -487,28 +538,28 @@ class _ScopeVisitor(object):
         level = 0
         if node.level:
             level = node.level
-        imported_module = pynames.ImportedModule(self.get_module(),
-                                                 node.module, level)
+        imported_module = pynames.ImportedModule(self.get_module(), node.module, level)
         if self._is_ignored_import(imported_module):
             return
-        if len(node.names) == 1 and node.names[0].name == '*':
+        if len(node.names) == 1 and node.names[0].name == "*":
             if isinstance(self.owner_object, PyModule):
-                self.owner_object.star_imports.append(
-                    StarImport(imported_module))
+                self.owner_object.star_imports.append(StarImport(imported_module))
         else:
             for imported_name in node.names:
                 imported = imported_name.name
                 alias = imported_name.asname
                 if alias is not None:
                     imported = alias
-                self.names[imported] = pynames.ImportedName(imported_module,
-                                                            imported_name.name)
+                self.names[imported] = pynames.ImportedName(
+                    imported_module, imported_name.name
+                )
 
     def _is_ignored_import(self, imported_module):
-        if not self.pycore.project.prefs.get('ignore_bad_imports', False):
+        if not self.pycore.project.prefs.get("ignore_bad_imports", False):
             return False
-        return not isinstance(imported_module.get_object(),
-                              rope.base.pyobjects.AbstractModule)
+        return not isinstance(
+            imported_module.get_object(), rope.base.pyobjects.AbstractModule
+        )
 
     def _Global(self, node):
         module = self.get_module()
@@ -521,14 +572,25 @@ class _ScopeVisitor(object):
             self.names[name] = pyname
 
 
-class _GlobalVisitor(_ScopeVisitor):
+class _ComprehensionVisitor(_ScopeVisitor):
+    def _comprehension(self, node):
+        ast.walk(node.target, self)
+        ast.walk(node.iter, self)
 
+    def _Name(self, node):
+        if isinstance(node.ctx, ast.Store):
+            self.names[node.id] = self._get_pyobject(node)
+
+    def _get_pyobject(self, node):
+        return pynames.AssignedName(lineno=node.lineno, module=self.get_module())
+
+
+class _GlobalVisitor(_ScopeVisitor):
     def __init__(self, pycore, owner_object):
         super(_GlobalVisitor, self).__init__(pycore, owner_object)
 
 
 class _ClassVisitor(_ScopeVisitor):
-
     def __init__(self, pycore, owner_object):
         super(_ClassVisitor, self).__init__(pycore, owner_object)
 
@@ -545,7 +607,6 @@ class _ClassVisitor(_ScopeVisitor):
 
 
 class _FunctionVisitor(_ScopeVisitor):
-
     def __init__(self, pycore, owner_object):
         super(_FunctionVisitor, self).__init__(pycore, owner_object)
         self.returned_asts = []
@@ -562,7 +623,6 @@ class _FunctionVisitor(_ScopeVisitor):
 
 
 class _ClassInitVisitor(_AssignVisitor):
-
     def __init__(self, scope_visitor, self_name):
         super(_ClassInitVisitor, self).__init__(scope_visitor)
         self.self_name = self_name
@@ -570,16 +630,17 @@ class _ClassInitVisitor(_AssignVisitor):
     def _Attribute(self, node):
         if not isinstance(node.ctx, ast.Store):
             return
-        if isinstance(node.value, ast.Name) and \
-           node.value.id == self.self_name:
+        if isinstance(node.value, ast.Name) and node.value.id == self.self_name:
             if node.attr not in self.scope_visitor.names:
                 self.scope_visitor.names[node.attr] = pynames.AssignedName(
-                    lineno=node.lineno, module=self.scope_visitor.get_module())
+                    lineno=node.lineno, module=self.scope_visitor.get_module()
+                )
             if self.assigned_ast is not None:
                 pyname = self.scope_visitor.names[node.attr]
                 if isinstance(pyname, pynames.AssignedName):
                     pyname.assignments.append(
-                        pynames.AssignmentValue(self.assigned_ast))
+                        pynames.AssignmentValue(self.assigned_ast)
+                    )
 
     def _Tuple(self, node):
         if not isinstance(node.ctx, ast.Store):
@@ -604,7 +665,6 @@ class _ClassInitVisitor(_AssignVisitor):
 
 
 class StarImport(object):
-
     def __init__(self, imported_module):
         self.imported_module = imported_module
 
@@ -612,6 +672,6 @@ class StarImport(object):
         result = {}
         imported = self.imported_module.get_object()
         for name in imported:
-            if not name.startswith('_'):
+            if not name.startswith("_"):
                 result[name] = pynames.ImportedName(self.imported_module, name)
         return result
